@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_video.c,v 1.49 2004/11/25 15:47:49 pmandin Exp $";
+ "@(#) $Id: SDL_video.c,v 1.52 2005/06/12 16:12:55 pmandin Exp $";
 #endif
 
 /* The high-level video driver subsystem */
@@ -212,6 +212,8 @@ int SDL_VideoInit (const char *driver_name, Uint32 flags)
 	video->offset_x = 0;
 	video->offset_y = 0;
 	memset(&video->info, 0, (sizeof video->info));
+	
+	video->displayformatalphapixel = NULL;
 
 	/* Set some very sane GL defaults */
 	video->gl_config.driver_loaded = 0;
@@ -460,33 +462,29 @@ static int SDL_GetVideoMode (int *w, int *h, int *BitsPerPixel, Uint32 flags)
 	SDL_closest_depths[table][0] = *BitsPerPixel;
 	SDL_closest_depths[table][7] = SDL_VideoSurface->format->BitsPerPixel;
 	for ( b = 0; !supported && SDL_closest_depths[table][b]; ++b ) {
+		int best;
+
 		format.BitsPerPixel = SDL_closest_depths[table][b];
 		sizes = SDL_ListModes(&format, flags);
 		if ( sizes == (SDL_Rect **)0 ) {
 			/* No sizes supported at this bit-depth */
 			continue;
 		}
+		best=0;
 		for ( i=0; sizes[i]; ++i ) {
-			if ((sizes[i]->w < *w) || (sizes[i]->h < *h)) {
-				if ( i > 0 ) {
-					--i;
-					*w = sizes[i]->w;
-					*h = sizes[i]->h;
-					*BitsPerPixel = SDL_closest_depths[table][b];
+			/* Mode with both dimensions bigger or equal than asked ? */
+			if ((sizes[i]->w >= *w) && (sizes[i]->h >= *h)) {
+				/* Mode with any dimension smaller or equal than current best ? */
+				if ((sizes[i]->w <= sizes[best]->w) || (sizes[i]->h <= sizes[best]->h)) {
+					best=i;
 					supported = 1;
-				} else {
-					/* Largest mode too small... */;
 				}
-				break;
 			}
 		}
-		if ( (i > 0) && ! sizes[i] ) {
-			/* The smallest mode was larger than requested, OK */
-			--i;
-			*w = sizes[i]->w;
-			*h = sizes[i]->h;
+		if (supported) {
+			*w=sizes[best]->w;
+			*h=sizes[best]->h;
 			*BitsPerPixel = SDL_closest_depths[table][b];
-			supported = 1;
 		}
 	}
 	if ( ! supported ) {
@@ -743,9 +741,7 @@ SDL_Surface * SDL_SetVideoMode (int width, int height, int bpp, Uint32 flags)
 	/* Load GL symbols (before MakeCurrent, where we need glGetString). */
 	if ( flags & (SDL_OPENGL | SDL_OPENGLBLIT) ) {
 
-#if (defined(macintosh) && !defined(__MWERKS__))
-#define __SDL_NOGETPROCADDR__
-#elif defined(__QNXNTO__) && (_NTO_VERSION < 630)
+#if defined(__QNXNTO__) && (_NTO_VERSION < 630)
 #define __SDL_NOGETPROCADDR__
 #elif defined(__MINT__)
 #define __SDL_NOGETPROCADDR__
