@@ -1,6 +1,6 @@
 /*
 	SDL - Simple DirectMedia Layer
-	Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002  Sam Lantinga
+    Copyright (C) 1997-2004 Sam Lantinga
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_dgavideo.c,v 1.11 2002/08/20 16:58:49 slouken Exp $";
+ "@(#) $Id: SDL_dgavideo.c,v 1.15 2004/11/12 21:25:41 slouken Exp $";
 #endif
 
 /* DGA 2.0 based SDL video driver implementation.
@@ -284,15 +284,15 @@ static int cmpmodes(const void *va, const void *vb)
     /* Prefer DirectColor visuals for otherwise equal modes */
     if ( (a->viewportWidth == b->viewportWidth) &&
          (b->viewportHeight == a->viewportHeight) ) {
-         if ( a->visualClass == DirectColor )
-             return -1;
-         if ( b->visualClass == DirectColor )
-             return 1;
-         return 0;
-    } else {
-        if(a->viewportWidth > b->viewportWidth)
+        if ( a->visualClass == DirectColor )
             return -1;
+        if ( b->visualClass == DirectColor )
+            return 1;
+        return 0;
+    } else if ( a->viewportWidth == b->viewportWidth ) {
         return b->viewportHeight - a->viewportHeight;
+    } else {
+        return b->viewportWidth - a->viewportWidth;
     }
 }
 static void UpdateHWInfo(_THIS, SDL_NAME(XDGAMode) *mode)
@@ -474,7 +474,7 @@ SDL_Surface *DGA_SetVideoMode(_THIS, SDL_Surface *current,
 		SDL_SetError("Unable to switch to requested mode");
 		return(NULL);
 	}
-	DGA_visualClass = modes[i].visualClass;
+	DGA_visualClass = mode->mode.visualClass;
 	memory_base = (Uint8 *)mode->data;
 	memory_pitch = mode->mode.bytesPerScanline;
 
@@ -816,7 +816,9 @@ static int DGA_FillHWRect(_THIS, SDL_Surface *dst, SDL_Rect *rect, Uint32 color)
   printf("Hardware accelerated rectangle fill: %dx%d at %d,%d\n", w, h, x, y);
 #endif
 	SDL_NAME(XDGAFillRectangle)(DGA_Display, DGA_Screen, x, y, w, h, color);
-	XFlush(DGA_Display);
+	if ( !(this->screen->flags & SDL_DOUBLEBUF) ) {
+		XFlush(DGA_Display);
+	}
 	DGA_AddBusySurface(dst);
 	UNLOCK_DISPLAY();
 	return(0);
@@ -856,7 +858,9 @@ static int HWAccelBlit(SDL_Surface *src, SDL_Rect *srcrect,
 		SDL_NAME(XDGACopyArea)(DGA_Display, DGA_Screen,
 			srcx, srcy, w, h, dstx, dsty);
 	}
-	XFlush(DGA_Display);
+	if ( !(this->screen->flags & SDL_DOUBLEBUF) ) {
+		XFlush(DGA_Display);
+	}
 	DGA_AddBusySurface(src);
 	DGA_AddBusySurface(dst);
 	UNLOCK_DISPLAY();
