@@ -95,6 +95,7 @@ static int  QZ_FillHWRect (_THIS, SDL_Surface *dst, SDL_Rect *rect, Uint32 color
 #endif
 static int  QZ_LockHWSurface(_THIS, SDL_Surface *surface);
 static void QZ_UnlockHWSurface(_THIS, SDL_Surface *surface);
+static int QZ_AllocHWSurface(_THIS, SDL_Surface *surface);
 static void QZ_FreeHWSurface (_THIS, SDL_Surface *surface);
 /* static int  QZ_FlipHWSurface (_THIS, SDL_Surface *surface); */
 
@@ -137,6 +138,7 @@ static SDL_VideoDevice* QZ_CreateDevice (int device_index) {
 
     device->LockHWSurface   = QZ_LockHWSurface;
     device->UnlockHWSurface = QZ_UnlockHWSurface;
+    device->AllocHWSurface   = QZ_AllocHWSurface;
     device->FreeHWSurface   = QZ_FreeHWSurface;
     /* device->FlipHWSurface   = QZ_FlipHWSurface */;
 
@@ -639,6 +641,9 @@ static SDL_Surface* QZ_SetVideoFullScreen (_THIS, SDL_Surface *current, int widt
     /* Save the flags to ensure correct tear-down */
     mode_flags = current->flags;
 
+    /* we're fullscreen, so flag all input states... */
+    SDL_PrivateAppActive(1, SDL_APPMOUSEFOCUS | SDL_APPINPUTFOCUS | SDL_APPACTIVE);
+
     return current;
 
     /* Since the blanking window covers *all* windows (even force quit) correct recovery is crucial */
@@ -1503,6 +1508,10 @@ static void QZ_UnlockHWSurface(_THIS, SDL_Surface *surface) {
 
 }
 
+static int QZ_AllocHWSurface(_THIS, SDL_Surface *surface) {
+    return(-1); /* unallowed (no HWSURFACE support here). */
+}
+
 static void QZ_FreeHWSurface (_THIS, SDL_Surface *surface) {
 }
 
