@@ -22,7 +22,7 @@
 
 #ifdef SAVE_RCSID
 static char rcsid =
- "@(#) $Id: SDL_x11modes.c,v 1.21 2004/01/04 16:49:27 slouken Exp $";
+ "@(#) $Id: SDL_x11modes.c,v 1.22 2004/11/12 21:25:42 slouken Exp $";
 #endif
 
 /* Utilities for getting and setting the X display mode */
@@ -85,9 +85,10 @@ static int cmpmodes(const void *va, const void *vb)
 {
     const SDL_NAME(XF86VidModeModeInfo) *a = *(const SDL_NAME(XF86VidModeModeInfo)**)va;
     const SDL_NAME(XF86VidModeModeInfo) *b = *(const SDL_NAME(XF86VidModeModeInfo)**)vb;
-    if( (a->vdisplay > b->vdisplay) && (a->hdisplay >= b->hdisplay) )
-        return -1;
-    return b->hdisplay - a->hdisplay;
+    if ( a->hdisplay == b->hdisplay )
+        return b->vdisplay - a->vdisplay;
+    else
+        return b->hdisplay - a->hdisplay;
 }
 #endif
 
